#--------------------------------------------------------------------------------
# YongHan Lee
# November, 2006.
# Scenario B
#
#    ----                                                         ---------
#    |s1|-----------                                   -----------| dest1 |
#    ----           \                                 /           ---------
#                    \                               /   
#                     \----                     ----/           
#                      |e1|---------------------|e2|
#                     /----                     ----\    
#                    /                               \  
#    ----           /                                 \           ---------
#    |s2|-----------                                   -----------| dest2 |
#    ----                                                         ---------
#
#--------------------------------------------------------------------------------


set ns [new Simulator]
set testTime 402.0

Agent/TCP set window_ 64

if {$argc < 5} {
        puts stderr "ns tfrc_fc_scenario_b.tcl \[x (bottleneck in Mbit/s)\] \[Sack|Newreno (TCP version)\] \[nb of competitive TCP flow\] \[nb of TFRC FC flow\] \[d Delay (in ms)\]"
        exit 1
}

set C [lindex $argv 0]
set tcpver [lindex $argv 1]
set nbtcp [lindex $argv 2]
set nbtfrc [lindex $argv 3]
set D [lindex $argv 4]

Agent/TFRC_FCSink set NumFeedback_ 2
Agent/TFRC_FCSink set applReadRate_ 0

# Create a nam trace datafile.
#set namfile [open out.nam w]
#$ns namtrace-all $namfile

#open the trace file
set tracefile [open temp/out_tfrc_fc_varyRR.tr w]
$ns trace-all $tracefile

proc finish {} {
   global ns namfile tracefile
   $ns flush-trace
   close $tracefile
   exit 0
}

# Set up the network topology shown at the top of this file:
set s1 [$ns node]
set s2 [$ns node]
set e1 [$ns node]
set e2 [$ns node]
set dest1 [$ns node]
set dest2 [$ns node]

$ns duplex-link $s1 $e1 10Mb 0ms DropTail
$ns duplex-link $s2 $e1 10Mb 0ms DropTail

$ns duplex-link $e1 $e2 $C.Mb $D.ms DropTail

$ns duplex-link $e2 $dest1 10Mb 0ms DropTail
$ns duplex-link $e2 $dest2 10Mb 0ms DropTail

# TFRC
for {set i 0} {$i<$nbtfrc} {incr i} {
   set ftptfrc($i) [$ns create-connection TFRC_FC $s2 TFRC_FCSink $dest2 0]
   $ns at 100.0 "$ftptfrc($i) change 100"
   $ns at 200.0 "$ftptfrc($i) change 700"
   $ns at 300.0 "$ftptfrc($i) change 300"
}

# TCP
for {set j 0} {$j<$nbtcp} {incr j} {
  if {$tcpver == "Sack"} {
  	set tcp($j) [new Agent/TCP/FullTcp/Sack]
  	set ftptcp($j) [new Application/FTP]
  	$ftptcp($j) attach-agent $tcp($j)
  	set sinktcp($j) [new Agent/TCP/FullTcp/Sack]
  	$ns attach-agent $s1 $tcp($j)
  	$ns attach-agent $dest1 $sinktcp($j)
  	#$tcp1 set window_ $win
  	#$sink set window_ $win
  	$tcp($j) set fid_ 1
  	$sinktcp($j) set fid_ 1
  	$ns connect $tcp($j) $sinktcp($j)
  	# set up TCP-level connections
  	$sinktcp($j) listen ; # will figure out who its peer is
  } else {
  	set agentNR($j) [new Agent/TCP/Newreno]
  	$ns attach-agent $s1 $agentNR($j)
  	set ftptcp($j) [new Application/FTP]
  	$ftptcp($j) attach-agent $agentNR($j)
  	set agentSink($j) [new Agent/TCPSink]
  	$ns attach-agent $dest1 $agentSink($j)
  	$ns connect $agentNR($j) $agentSink($j)
  }
}

if {$tcpver == "mix_tfrc"} {
   #Mix TFRC and TFRC FC
   set ftptfrc_1 [$ns create-connection TFRC $s1 TFRCSink $dest1 0]
   $ns at 0.3 "$ftptfrc_1 start"
   $ns at $testTime "$ftptfrc_1 stop"
}

for {set j 0} {$j<$nbtcp} {incr j} {
    $ns at 0.3 "$ftptcp($j) start"
    $ns at $testTime "$ftptcp($j) stop"
}
for {set i 0} {$i<$nbtfrc} {incr i} {
    $ns at 1.0 "$ftptfrc($i) start"
    $ns at [expr $testTime - 10.0] "$ftptfrc($i) stop"
}

$ns at [expr $testTime + 1.0] "finish"

$ns run
